/*
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the LAM/MPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * LAM/MPI source distribution.
 *
 * $Id: big.c,v 1.5.2.1 2002/10/14 18:46:19 brbarret Exp $
 *
 * Test some long, big messages.
 */

#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

#include "lamtest_error.h"

#define SIZE 1048576


int
main(int argc, char* argv[])
{
  double *buf;
  int i, tag = 200;
  int my_rank, size;
  int num = 0;
  MPI_Request *reqs;

  MPI_Init(&argc, &argv);
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);

  buf = malloc(SIZE * size * 2 * sizeof(double));
  reqs = malloc(sizeof(MPI_Request) * size * 2);

  /* All to all pattern, long messages.  Should not deadlock. */

  for (i = 0; i < size; i++)
    if (i != my_rank)
      MPI_Isend(&buf[i * size], SIZE, MPI_DOUBLE, i, tag, MPI_COMM_WORLD, 
		&reqs[num++]);
  
  for (i = 0; i < size; i++)
    if (i != my_rank)
      MPI_Irecv(&buf[i * size * 2], SIZE, MPI_DOUBLE, i, tag, MPI_COMM_WORLD,
		&reqs[num++]);

  MPI_Waitall(num, reqs, MPI_STATUSES_IGNORE);
  free(buf);
  free(reqs);
  
  MPI_Finalize();
  return 0;
}
