/*
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the LAM/MPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * LAM/MPI source distribution.
 *
 * $Id: ring.c,v 1.4.2.2 2002/10/14 18:46:19 brbarret Exp $
 *
 * Test some long, big messages in a ring pattern.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "mpi.h"

#include "lamtest_error.h"


int 
main(int argc, char **argv)
{
  int num; 
  int rank, size, tag, next, from;
  int *buffer = 0;
  int bufsize = 1048576;

  /* Start up MPI */

  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &size);
 
  /* Arbitrarily choose 201 to be our tag.  Calculate the rank of the
     next process in the ring.  Use the modulus operator so that the
     last process "wraps around" to rank zero.  Use MPI_ANY_SOURCE
     just to make the test a bit more difficult.  */

  tag = 201;
  next = (rank + 1) % size;
  from = MPI_ANY_SOURCE;

  buffer = (int*) malloc(bufsize * sizeof(int));
  memset(buffer, 0, bufsize * sizeof(int));

  /* If we are the "console" process, get a integer from the */
  /* user to specify how many times we want to go around the */
  /* ring */

  if (rank == 0) {
    /* For simplicity, we'll just hardwire in a value for now */
    num = 5;

    buffer[0] = num;
    MPI_Send(buffer, bufsize, MPI_INT, next, tag, MPI_COMM_WORLD); 
  }

  /* Pass the message around the ring.  The exit mechanism works */
  /* as follows: the message (a positive integer) is passed */
  /* around the ring.  Each time is passes rank 0, it is decremented. */
  /* When each processes receives the 0 message, it passes it on */
  /* to the next process and then quits.  By passing the 0 first, */
  /* every process gets the 0 message and can quit normally. */

  while (1) {
    MPI_Recv(buffer, bufsize, MPI_INT, from, tag, MPI_COMM_WORLD, 
	     MPI_STATUS_IGNORE);
    num = buffer[0];

    if (rank == 0) {
      num--;
      buffer[0]--;
    }

    MPI_Send(buffer, bufsize, MPI_INT, next, tag, MPI_COMM_WORLD);

    if (num == 0)
      break;
  }

  /* Do the final recv */

  if (rank == 0)
    MPI_Recv(buffer, bufsize, MPI_INT, from, tag, MPI_COMM_WORLD, 
	     MPI_STATUS_IGNORE);

  /* Quit */

  free(buffer);
  MPI_Finalize();
  return 0;
}
