/*
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the LAM/MPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * LAM/MPI source distribution.
 *
 * $Id: put.c,v 1.6.2.1 2002/10/14 18:46:20 brbarret Exp $
 *
 * Basic get/put test.  Mainly contributed by some LAM user(sorry;
 * I've long since lost the name of the original author!).  Somewhat
 * modified to be suitable for the LAM test suite.  
 */

#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

#include "lamtest_error.h"


int 
main(int argc, char *argv[])
{
  int size, rank;
  int target;
  int *winArea;
  MPI_Win win;

  MPI_Init(&argc, &argv);

  /*  Enough processes? */

  MPI_Comm_size(MPI_COMM_WORLD, &size);
  if (size != 2)
    lamtest_fatal_error(__FILE__, __LINE__, "Need only 2 processes.\n");
  target = (size - 1);

  /* Make window */

  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Alloc_mem(sizeof(int), MPI_INFO_NULL, &winArea);
  MPI_Win_create(winArea, sizeof(int), sizeof(int), MPI_INFO_NULL, 
		 MPI_COMM_WORLD, &win);

  /* Assign the window on the target, and ensure that everyone waits
     for it to be assigned */

  if (rank == target)
    *winArea = -1;
  MPI_Win_fence(0, win);

  /* Now on rank 0, put that value to the target */

  if (rank == 0)
    MPI_Put(&target, 1, MPI_INT, 1, 0, 1, MPI_INT, win);
  MPI_Win_fence(0, win);

  /* Check to see that we got the right value */

  if (rank == target)
    if (*winArea != target)
      lamtest_error(__FILE__, __LINE__, "ERROR: Rank %d got value of %d when expecting a value of %d\n",
		    target, *winArea, target);

  MPI_Win_free(&win);  
  MPI_Free_mem(winArea);
  MPI_Finalize();
  return(0);
}
